local noise = require("perlin")

function on_mod_init( event )
    global.noise_permutation = noise.generate_permutation()
    global.world_index_for_player = {}
    global.worlds = {}
    global.worlds[0] = "nauvis"
    for i = -2, 2 do
        if i ~= 0 then
            local world = create_world(i)
            game.surfaces[world].request_to_generate_chunks({0,0}, 5)
        end
    end
end

function on_mod_load( event )
    noise.set_permutation(global.noise_permutation)
end

function on_player_created( event )
    global.world_index_for_player[event.player_index] = 0
    local player = game.players[event.player_index]
    player.insert{
        name = "stepper",
        count = 1
   }
end

function on_tick( event )
    for index, player in pairs(game.players) do
        local stepper_panel = player.gui.left["stepper-panel"] 
        if stepper_panel ~= nil then
            local soft_place_value = get_soft_place_value_at(player.position, player.surface)
            if soft_place_value < 0 then soft_place_value = 0 end
            if soft_place_value > 1 then soft_place_value = 1 end
            stepper_panel["soft-place-value"].value = soft_place_value
            local offset = get_soft_place_step_offset_at(player.position, player.surface)
            if offset > 1 then
                stepper_panel["soft-place"].caption = "Soft place found."
            else
                stepper_panel["soft-place"].caption = ""
            end
        end
    end
end

function get_soft_place_value_at( position, surface )
    local p = {
        x = position.x + surface.map_gen_settings.seed,
        y = position.y + surface.map_gen_settings.seed
    }
    return (noise.fbm2d(p.x, p.y, 2, 0.012, 1, 0.75) + 1) * 0.5
end

function get_soft_place_step_offset_at( position, surface )
    if get_soft_place_value_at(position, surface) >= 1.0 then
        local p2 = {
            x = position.x + 919,
            y = position.y + 929
        }
        local val = (noise.fbm2d(p2.x, p2.y, 1, 0.001, 1, 0.75) + 1) * 0.5
        local offset = (math.floor(val * 10) * 2) + 1
        return offset
    else
        return 1
    end
end

function on_player_cursor_stack_changed( event )
    local player = game.players[event.player_index]
    local stack = player.cursor_stack
    if stack.valid_for_read and stack.name == "stepper" then
        show_stepper_gui( player )
    elseif gui_contains_child(player.gui.left, "stepper-panel") then
        player.gui.left["stepper-panel"].destroy()
    end
end

function on_gui_click( event )
    local el = event.element
    local player = game.players[event.player_index]
    local step_offset = get_soft_place_step_offset_at(player.position, player.surface)
    if el.name == "step-east" then
        step_into_world(player, -step_offset)
    elseif el.name == "step-west" then
        step_into_world(player, step_offset)
    end
end

function show_stepper_gui( player )
    local panel = player.gui.left.add{
        type = "frame",
        name = "stepper-panel",
        caption = "Stepper",
        direction = "vertical"
    }
    panel.add{
        type = "label",
        name = "world-label",
        caption = current_world_name(player)
    }
    panel.add{
        type = "label",
        name = "soft-place",
        caption = ""
    }
    panel.add{
        type = "progressbar",
        name = "soft-place-value",
        value = 0,
        size = 100
    }
    local button_flow = panel.add{
        type = "flow",
        name = "buttons",
        direction = "horizontal"
    }
    button_flow.add{
        type = "button",
        name = "step-east",
        caption = "Step East"
    }
    button_flow.add{
        type = "button",
        name = "step-west",
        caption = "Step West"
    }
end

function gui_contains_child( el, child_name )
    for i, name in ipairs(el.children_names) do
        if name == child_name then
            return true
        end
    end
    return false
end

function table.pick( t )
    return t[math.floor((math.random()*#t))]
end

function step_into_world( player, offset )
    local current_world_index = global.world_index_for_player[player.index]
    local next_world_index = current_world_index + offset
    local next_world_name = global.worlds[next_world_index]
    if next_world_name == nil then
        next_world_name = create_world(next_world_index)
        game.surfaces[next_world_name].request_to_generate_chunks(player.position, 5)
    end
    local teleportPos = game.surfaces[next_world_name].find_non_colliding_position("player", player.position, 5, 1)
    if teleportPos ~= nil or offset > 1 or offset < -1 then
        drop_iron(player)
        player.character.damage(25, player.force)
        player.teleport(teleportPos or player.position, next_world_name)
        global.world_index_for_player[player.index] = next_world_index
        if player.gui.left["stepper-panel"] ~= nil then
            player.gui.left["stepper-panel"]["world-label"].caption = current_world_name(player)
        end
        if global.worlds[next_world_index-1] == nil then
            local new_world = create_world(next_world_index-1)
            game.surfaces[new_world].request_to_generate_chunks(player.position, 5)
        end
        if global.worlds[next_world_index+1] == nil then
            local new_world = create_world(next_world_index+1)
            game.surfaces[new_world].request_to_generate_chunks(player.position, 5)
        end
    else
        player.print("Cannot step due to obstruction.")
    end
end

function drop_iron( player )
    local iron_items = {}
    local inventories = {
        defines.inventory.player_main, defines.inventory.player_quickbar, defines.inventory.player_guns, 
        defines.inventory.player_ammo, defines.inventory.player_armor, defines.inventory.player_tools
    }
    for i, inv_key in ipairs(inventories) do
        local inv = player.get_inventory(inv_key)
        for name, count in pairs(inv.get_contents()) do
            local stack = {name = name, count = count}
            if item_is_iron(name) then
                inv.remove(stack)
                table.insert(iron_items, stack)
            elseif player.force.recipes[name] ~= nil and recipe_has_iron(player.force.recipes[name], player.force, 0) then
                inv.remove(stack)
                table.insert(iron_items, stack)
            end
        end
    end
    for i, stack in ipairs(iron_items) do
        player.surface.spill_item_stack(player.position, stack)
    end
end

function item_is_iron( name )
    return name == "iron-plate" or name == "iron-ore"
end

function recipe_has_iron( recipe, force, depth )
    if recipe == nil or depth > 10 then
        return false
    end
    for i, ingredient in ipairs(recipe.ingredients) do
        if item_is_iron(ingredient.name) then
            return true
        elseif recipe_has_iron(force.recipes[ingredient.name], force, depth + 1) then
            return true
        end
    end
end

function create_world( world_index )
    local world_name = "?"
    local world_abs_index = tostring(math.abs(world_index))
    if world_index > 0 then world_name = "w"..world_abs_index end
    if world_index < 0 then world_name = "e"..world_abs_index end
    game.create_surface(world_name, create_world_gen_settings())
    global.worlds[world_index] = world_name
    return world_name
end

function current_world_name( player )
    local world_index = global.world_index_for_player[player.index]
    local world_name = global.worlds[world_index]
    return world_name
end

function create_world_gen_settings()
    local mapGenFreq = {"none", "very-low", "low", "normal", "high", "very-high"}
    local mapGenSize = {"none", "very-small", "medium", "big", "very-big"}
    local mapGenRichness = {"very-poor", "poor", "regular", "good", "very-good"}
    local is_peaceful = (math.random() < 0.03)
    local placables = {"iron-ore", "copper-ore", "stone", "crude-oil", "coal", "enemy-base"}
    local autoplace = {}
    for i, placable in ipairs(placables) do
        if math.random() > 0.10 then
            autoplace[placable] = {
                frequency = table.pick(mapGenFreq),
                size = table.pick(mapGenSize),
                richness = table.pick(mapGenRichness)
            }
        end
    end
    local settings = {
        terrain_segmentation = table.pick(mapGenFreq),
        autoplace_controls = autoplace,
        water = table.pick(mapGenSize),
        seed = math.floor(math.random() * 100000),
        shift = {math.random()*1000, math.random()*1000},
        peaceful_mode = is_peaceful
    }
    return settings
end

script.on_init(on_mod_init)
script.on_load(on_mod_load)
script.on_event(defines.events.on_player_created, on_player_created)
script.on_event(defines.events.on_tick, on_tick)
script.on_event(defines.events.on_player_cursor_stack_changed, on_player_cursor_stack_changed)
script.on_event(defines.events.on_gui_click, on_gui_click)