-- Perlin noise and more in pure Lua.
-- Found this on stackoverflow but can't find the URL anymore.
local noise = {}

local p = {}
local permutation = {}

local function fade(t)
  return t * t * t * (t * (t * 6 - 15) + 10)
end

local function lerp(t, a, b)
  return a + t * (b - a)
end

local function grad(hash, x, y, z)
  local h, u, v = hash % 16
  if h < 8 then u = x else u = y end
  if h < 4 then v = y elseif h == 12 or h == 14 then v = x else v = z end
  local r
  if h % 2 == 0 then r = u else r = -u end
  if h % 4 == 0 then r = r + v else r = r - v end
  return r
end

function noise.generate_permutation()
    local pt = {}
    for i = 0, 255 do table.insert(pt, i) end
    for i = #pt, 1, -1 do
        local pos = math.random(1, i)
        local tmp = pt[pos]
        pt[pos] = pt[i]
        pt[i] = tmp
    end
    permutation = pt
    for i = 0, 255 do
        p[i] = permutation[i + 1]
        p[256 + i] = permutation[i + 1]
    end
    return permutation
end

function noise.set_permutation( pt )
    permutation = pt
    for i = 0, 255 do
        p[i] = permutation[i + 1]
        p[256 + i] = permutation[i + 1]
    end
end

function noise.perlin2d(x, y)
    return noise.perlin(x, y, 0)
end

function noise.perlin(x, y, z) 
  y = y or 0
  z = z or 0
  local X = math.floor(x % 255)
  local Y = math.floor(y % 255)
  local Z = math.floor(z % 255)
  x = x - math.floor(x)
  y = y - math.floor(y)
  z = z - math.floor(z)
  local u = fade(x)
  local v = fade(y)
  local w = fade(z)

  A   = p[X    ] + Y
  AA  = p[A    ] + Z
  AB  = p[A + 1] + Z
  B   = p[X + 1] + Y
  BA  = p[B    ] + Z
  BB  = p[B + 1] + Z

  return lerp(w, lerp(v, lerp(u, grad(p[AA    ], x    , y    , z    ), 
                                 grad(p[BA    ], x - 1, y    , z    )), 
                         lerp(u, grad(p[AB    ], x    , y - 1, z    ), 
                                 grad(p[BB    ], x - 1, y - 1, z    ))),
                 lerp(v, lerp(u, grad(p[AA + 1], x    , y    , z - 1),  
                                 grad(p[BA + 1], x - 1, y    , z - 1)),
                         lerp(u, grad(p[AB + 1], x    , y - 1, z - 1),
                                 grad(p[BB + 1], x - 1, y - 1, z - 1))))
end

function noise.fbm(x, y, z, octaves, freq, lacunarity, gain)
  octaves = octaves or 8
  lacunarity = lacunarity or 2
  gain = gain or 0.5
  local amplitude = 1.0
  local frequency = freq or 1.0
  local sum = 0.0
  for i = 0, octaves do
    sum = sum + amplitude * noise.perlin(x * frequency, y * frequency, z * frequency)
    amplitude = amplitude * gain
    frequency = frequency * lacunarity
  end
  return sum
end

function noise.fbm2d(x, y, octaves, freq, lacunarity, gain)
    return noise.fbm(x, y, 0, octaves, freq, lacunarity, gain)
end

return noise