-- debug_status = 1
debug_mod_name = "WagonCapacityColor"
debug_file = debug_mod_name .. "-debug.txt"
require("utils")
require("config")

--------------------------------------------------------------------------------------
local function retrieve_wagons()
	-- retrieve existing wagons
	
	global.wagons = {}
	
	for _, surf in pairs(game.surfaces) do
		for _, wagon in pairs(surf.find_entities_filtered{type="cargo-wagon"}) do
			table.insert(global.wagons,wagon)
		end
	end
	
	debug_print( "wagons=", #global.wagons )
end

--------------------------------------------------------------------------------------
function init_globals()
	-- initialize or update general globals of the mod
	debug_print( "init_globals " )

	global.ticks = global.ticks or 0
	global.wagons = global.wagons or {}
end

--------------------------------------------------------------------------------------
local function on_init() 
	-- called once, the first time the mod is loaded on a game (new or existing game)
	debug_print( "on_init" )
	init_globals()
	retrieve_wagons()
end

script.on_init(on_init)

--------------------------------------------------------------------------------------
local function on_configuration_changed(data)
	-- detect any mod or game version change
	if data.mod_changes ~= nil then
		local changes = data.mod_changes[debug_mod_name]
		if changes ~= nil then
			debug_print( "update mod: ", debug_mod_name, " ", tostring(changes.old_version), " to ", tostring(changes.new_version) )
		
			init_globals()
		end
		
		-- if any other mod change (possible wagon type disappear...
		retrieve_wagons()
	end
end

script.on_configuration_changed(on_configuration_changed)

--------------------------------------------------------------------------------------
local function on_creation( event )
	local ent = event.created_entity
	
	if ent.type == "cargo-wagon" then
		debug_print( "creation ", ent.name )
		
		table.insert(global.wagons,ent)
	end
end

script.on_event(defines.events.on_built_entity, on_creation )
script.on_event(defines.events.on_robot_built_entity, on_creation )

--------------------------------------------------------------------------------------
local function on_destruction( event )
	local ent = event.entity
	
	if ent.type == "cargo-wagon" then
		debug_print( "destruction ", ent.name )
	
		for i, wagon in pairs(global.wagons) do
			if wagon == ent then
				table.remove( global.wagons, i )
				break
			end
		end
	end
end

script.on_event(defines.events.on_entity_died, on_destruction )
script.on_event(defines.events.on_robot_pre_mined, on_destruction )
script.on_event(defines.events.on_preplayer_mined_item, on_destruction )

--------------------------------------------------------------------------------------
local function on_tick(event)
	if global.ticks <= 0 then 
		global.ticks = 97
		
		for i, wagon in pairs(global.wagons) do
			if wagon.valid then
				local invent = wagon.get_inventory(1)
				if invent then
					if invent.is_empty() then
						wagon.color = color_empty
					else
						local full = true
						local max = invent.hasbar() and invent.getbar() or #invent
						for i=1,max do
							if invent[i].valid_for_read and invent.can_insert(invent[i]) then
								wagon.color = color_partial
								full = false
								break
							end
						end
						if full then wagon.color = color_full end
					end
				else
					wagon.color = {}
				end
			else
				table.remove( global.wagons, i )
			end
		end
	else
		global.ticks = global.ticks -1 
	end
end

script.on_event(defines.events.on_tick, on_tick)

--------------------------------------------------------------------------------------

local interface = {}

function interface.reset()
	debug_print( "reset" )
	
	init_globals()

	retrieve_wagons()
end

remote.add_interface( "wagoncolor", interface )

-- /c remote.call( "wagoncolor", "reset" )