require ("prototypes.entity.demo-spawner-animation")
require ("prototypes.entity.demo-biter-animations")
require ("prototypes.entity.demo-enemy-sounds")
require ("prototypes.entity.spitter-animations")

shadow_spawner_tint = {r=0.99, g=0.99, b=0.99, a=1}
shadow_biter_tint1 = {r=0, g=0, b=0, a=0}

data:extend(
{
  {
    type = "unit",
    name = "shadow-biter",
    icon = "__base__/graphics/icons/creeper.png",
    flags = {"placeable-player", "placeable-enemy", "placeable-off-grid", "breaths-air", "not-repairable"},
    max_health = 750,
    order="b-b-b",
    subgroup="enemies",
    resistances =
    {
      {
        type = "physical",
        percent = 85,
      },
      {
        type = "explosion",
        percent = 70
      },
      {
        type = "electric",
        percent = 90
      },
      {
        type = "laser",
        percent = 90
      },
      {
        type = "fire",
        percent = -400
      }
    },
    healing_per_tick = 0.01,
    collision_box = {{-0.3, -0.3}, {0.3, 0.3}},
    selection_box = {{-0.7, -1.5}, {0.7, 0.3}},
    sticker_box = {{-0.3, -0.5}, {0.3, 0.1}},
    distraction_cooldown = 300,
    attack_parameters =
    {
      type = "projectile",
      ammo_category = "melee",
      ammo_type = make_unit_melee_ammo_type(15),
      range = 1,
      cooldown = 35,
      sound = make_biter_roars(0.7),
      animation = biterattackanimation(mediumbiterscale, shadow_biter_tint1, shadow_biter_tint1)
    },
    vision_distance = 30,
    movement_speed = 0.185,
    distance_per_frame = 0.15,
    -- in pu
    pollution_to_join_attack = 1000,
    corpse = "medium-biter-corpse",
    dying_explosion = "blood-explosion-small",
    working_sound = make_biter_calls(0.8),
    dying_sound = make_biter_dying_sounds(1.0),
    run_animation = biterrunanimation(mediumbiterscale, shadow_biter_tint1, shadow_biter_tint1)
  },
  {
    type = "unit-spawner",
    name = "shadow-spawner",
    icon = "__base__/graphics/icons/biter-spawner.png",
    flags = {"placeable-player", "placeable-enemy", "not-repairable"},
    max_health = 700,
    order="b-b-h",
    subgroup="enemies",
    working_sound = {
      sound =
      {
        {
          filename = "__base__/sound/creatures/spawner.ogg",
          volume = 1.0
        }
      },
      apparent_volume = 2
    },
    dying_sound =
    {
      {
        filename = "__base__/sound/creatures/spawner-death-1.ogg",
        volume = 1.0
      },
      {
        filename = "__base__/sound/creatures/spawner-death-2.ogg",
        volume = 1.0
      }
    },
    resistances =
    {
      {
        type = "physical",
        decrease = 2,
      },
      {
        type = "explosion",
        decrease = 5,
        percent = 15,
      }
    },
    healing_per_tick = 0.02,
    collision_box = {{-3.2, -2.2}, {2.2, 2.2}},
    selection_box = {{-3.5, -2.5}, {2.5, 2.5}},
    pollution_absorbtion_absolute = 20,
    pollution_absorbtion_proportional = 0.01,
    corpse = "shadow-spawner-corpse",
    dying_explosion = "blood-explosion-huge",
    loot =
    {
      {
        count_max = 15,
        count_min = 5,
        item = "alien-artifact",
        probability = 1
      }
    },
    max_count_of_owned_units = 7,
    max_friends_around_to_spawn = 5,
    animations =
    {
      spawner_idle_animation(0, shadow_spawner_tint),
      spawner_idle_animation(1, shadow_spawner_tint),
      spawner_idle_animation(2, shadow_spawner_tint),
      spawner_idle_animation(3, shadow_spawner_tint)
    },

    result_units = (function()
                     local res = {}
                     res[1] = {"shadow-biter", {{0.0, 0.3}, {0.35, 0}}}
                     res[2] = {"shadow-biter", {{0.25, 0.0}, {0.5, 0.3}, {0.7, 0.0}}}
                     res[3] = {"shadow-biter", {{0.5, 0.0}, {0.7, 0.3}, {0.9, 0.1}}}
                     res[4] = {"shadow-biter", {{0.6, 0.0}, {1.0, 0.4}}}
                     res[5] = {"shadow-biter", {{0.99, 0.0}, {1.0, 0.3}}}
                     return res
                   end)(),
    -- With zero evolution the spawn rate is 6 seconds, with max evolution it is 2.5 seconds
    spawning_cooldown = {360, 150},
    spawning_radius = 10,
    spawning_spacing = 3,
    max_spawn_shift = 0,
    max_richness_for_spawn_shift = 100,
	call_for_help_radius = 50
  },

  {
    type = "corpse",
    name = "shadow-spawner-corpse",
    flags = {"placeable-neutral", "placeable-off-grid", "not-on-map", "not-on-map"},
    icon = "__base__/graphics/icons/biter-spawner-corpse.png",
    collision_box = {{-2, -2}, {2, 2}},
    selection_box = {{-2, -2}, {2, 2}},
    selectable_in_game = false,
    dying_speed = 0.04,
    time_before_removed = 15 * 60 * 60,
    subgroup="corpses",
    order = "c[corpse]-c[spitter-spawner]",
    final_render_layer = "corpse",
    animation =
    {
      spawner_die_animation(0, shadow_spawner_tint),
      spawner_die_animation(1, shadow_spawner_tint),
      spawner_die_animation(2, shadow_spawner_tint),
      spawner_die_animation(3, shadow_spawner_tint)
    }
  },
})
