local autoplace_utils = require("autoplace_utils")
local grass_vehicle_speed_modifier = 1.6

local function autoStone(noise_layer, rectangles)
  local ret = {}

  if noise_layer then
    ret = {
      {
        influence = 1e3 + 950000,
        noise_layer = noise_layer,
        noise_persistence = 1,
        octaves_difference = -1
      }
    }
  end

   autoplace_utils.peaks(rectangles, ret)

   return { peaks = ret }
  end

function lavaAuto(from_depth, rectangles)
  local ret =
  {
    {
      -- Water and deep water have absolute priority. We simulate this by giving
      -- them absurdly large influence
      influence = 1000000 + 1e3 + from_depth,
      elevation_optimal = -5000 - from_depth,
      elevation_range = 5000,
      elevation_max_range = 5000, -- everywhere below elevation 0 and nowhere else
    }
  }

  if rectangles == nil then
    ret[2] = { influence = 1 }
  end

  autoplace_utils.peaks(rectangles, ret)

  return { peaks = ret }
end

data:extend(
{
	-- vents-path
	{
		type = "recipe",
		name = "clean_floor",
		energy_required = 2,
		enabled = true,		
		ingredients = {{"iron-plate", 4},},
		result= "clean_floor",
		result_count = 8
	},

	{
		type = "item",
		name = "clean_floor",
		icon = "__base__/graphics/icons/concrete.png",
		flags = {"goes-to-main-inventory"},
		subgroup = "extraction-machine",
		order = "b[concrete]",
		stack_size = 500,
		place_as_tile =
		{
			result = "clean_floor",
			condition_size = 3,
			condition = { "water-tile" }
		}
	},
	
	{
		type = "tile",
		name = "clean_floor",
		needs_correction = false,
		minable = {hardness = 0.2, mining_time = 0.5, result = "clean_floor"},
		mined_sound = { filename = "__base__/sound/deconstruct-bricks.ogg" },
		collision_mask = {"ground-tile"},
		walking_speed_modifier = 1.4,
		layer = 61,
		variants =
		{
			main =
			{
				{
					picture = "__theUnderground__/graphics/tiles/clean_floor1.png",
					count = 1,
					size = 1
				}
			},
			inner_corner =
			{
				picture = "__base__/graphics/terrain/concrete/concrete-inner-corner.png",
				count = 8
			},
			outer_corner =
			{
				picture = "__base__/graphics/terrain/concrete/concrete-outer-corner.png",
				count = 8
			},
			side =
			{
				picture = "__base__/graphics/terrain/concrete/concrete-side.png",
				count = 8
			},
			u_transition =
			{
				picture = "__base__/graphics/terrain/concrete/concrete-u.png",
				count = 8
			},
			o_transition =
			{
				picture = "__base__/graphics/terrain/concrete/concrete-o.png",
				count = 1
			}
		},
		walking_sound =
		{
			{
				filename = "__base__/sound/walking/concrete-01.ogg",
				volume = 1.2
			},
			{
				filename = "__base__/sound/walking/concrete-02.ogg",
				volume = 1.2
			},
			{
				filename = "__base__/sound/walking/concrete-03.ogg",
				volume = 1.2
			},
			{
				filename = "__base__/sound/walking/concrete-04.ogg",
				volume = 1.2
			}
		},
		map_color={r=90, g=70, b=50},
		ageing=0,
		vehicle_friction_modifier = stone_path_vehicle_speed_modifier
	},
--World Tiles:
{
    type = "tile",
    name = "lavaStone",
    collision_mask = {"ground-tile", "doodad-layer"},
    --autoplace = autoStone("stone", {{{35, 0.8}, {0, 0.4}}}),
    layer = 20,
    variants =
    {
      main =
      {
        {
          picture = "__theUnderground__/graphics/terrain/lavaStone/lavaStone1.png",
          count = 16,
          size = 1,
        },
        {
          picture = "__theUnderground__/graphics/terrain/lavaStone/lavaStone2.png",
          count = 20,
          size = 2,
          probability = 0.91,
          weights = {0.150, 0.150, 0.150, 0.150, 0.018, 0.020, 0.015, 0.025, 0.015, 0.020, 0.025, 0.015, 0.025, 0.025, 0.010, 0.025, 0.020, 0.025, 0.025, 0.010 },
        },
        {
          picture = "__theUnderground__/graphics/terrain/lavaStone/lavaStone4.png",
          count = 20,
          line_length = 10,
          size = 4,
          probability = 1,
          weights = {0.100, 0.80, 0.80, 0.100, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01, 0.01 }
        }
      },
      inner_corner =
      {
        picture = "__theUnderground__/graphics/terrain/lavaStone/lavaStone-inner-corner.png",
        count = 8
      },
      outer_corner =
      {
        picture = "__theUnderground__/graphics/terrain/lavaStone/lavaStone-outer-corner.png",
        count = 8
      },
      side =
      {
        picture = "__theUnderground__/graphics/terrain/lavaStone/lavaStone-side.png",
        count = 8
      }
    },
    walking_sound =
    {
      {
        filename = "__base__/sound/walking/grass-01.ogg",
        volume = 0.8
      },
      {
        filename = "__base__/sound/walking/grass-02.ogg",
        volume = 0.8
      },
      {
        filename = "__base__/sound/walking/grass-03.ogg",
        volume = 0.8
      },
      {
        filename = "__base__/sound/walking/grass-04.ogg",
        volume = 0.8
      }
    },
    map_color={r=90, g=90, b=90},
    ageing=0.00045,
    vehicle_friction_modifier = grass_vehicle_speed_modifier
  },
  {
    type = "tile",
    name = "lava",
	    needs_correction = false,
    collision_mask =
    {
      "water-tile",
      "item-layer",
      "resource-layer",
      "player-layer",
      "doodad-layer"
    },
   -- autoplace = lavaAuto(0, {{{35, 1}, {15, 0.7}}}),
    layer = 40,
    variants =
    {
      main =
      {
        {
          picture = "__theUnderground__/graphics/terrain/lava/lava1.png",
          count = 8,
          size = 1
        },
        {
          picture = "__theUnderground__/graphics/terrain/lava/lava2.png",
          count = 8,
          size = 2
        },
        {
          picture = "__theUnderground__/graphics/terrain/lava/lava4.png",
          count = 6,
          size = 4
        }
      },
      inner_corner =
      {
        picture = "__theUnderground__/graphics/terrain/lava/lava-inner-corner.png",
        count = 6
      },
      outer_corner =
      {
        picture = "__theUnderground__/graphics/terrain/lava/lava-outer-corner.png",
        count = 6
      },
      side =
      {
        picture = "__theUnderground__/graphics/terrain/lava/lava-side.png",
        count = 8
      }
    },
    allowed_neighbors = {  "lavaStone" },
    map_color={r=45, g=0, b=0},
    ageing=0.0005
  },
	{
		type = "tile",
		name = "cracked_stone",
		needs_correction = false,
		collision_mask = {"ground-tile"},
		walking_speed_modifier = 1.4,
		layer = 62,
		variants =
		{
			main =
			{
				{
					picture = "__theUnderground__/graphics/tiles/clean_floor1.png",
					count = 1,
					size = 1
				}
			},
			inner_corner =
			{
				picture = "__base__/graphics/terrain/concrete/concrete-inner-corner.png",
				count = 8
			},
			outer_corner =
			{
				picture = "__base__/graphics/terrain/concrete/concrete-outer-corner.png",
				count = 8
			},
			side =
			{
				picture = "__base__/graphics/terrain/concrete/concrete-side.png",
				count = 8
			},
			u_transition =
			{
				picture = "__base__/graphics/terrain/concrete/concrete-u.png",
				count = 8
			},
			o_transition =
			{
				picture = "__base__/graphics/terrain/concrete/concrete-o.png",
				count = 1
			}
		},
		walking_sound =
		{
			{
				filename = "__base__/sound/walking/concrete-01.ogg",
				volume = 1.2
			},
			{
				filename = "__base__/sound/walking/concrete-02.ogg",
				volume = 1.2
			},
			{
				filename = "__base__/sound/walking/concrete-03.ogg",
				volume = 1.2
			},
			{
				filename = "__base__/sound/walking/concrete-04.ogg",
				volume = 1.2
			}
		},
		map_color={r=90, g=70, b=50},
		ageing=0,
		vehicle_friction_modifier = stone_path_vehicle_speed_modifier
	}
  })
